package ch.unisg.executorrobot.executor.adapter.out;
import java.io.IOException;
import java.net.URI;
import java.net.http.HttpClient;
import java.net.http.HttpRequest;
import java.net.http.HttpResponse;
import java.util.HashMap;
import java.util.Map;
import java.util.Optional;

import ch.unisg.ics.interactions.wot.td.ThingDescription;
import ch.unisg.ics.interactions.wot.td.affordances.ActionAffordance;
import ch.unisg.ics.interactions.wot.td.affordances.Form;
import ch.unisg.ics.interactions.wot.td.clients.TDHttpRequest;
import ch.unisg.ics.interactions.wot.td.clients.TDHttpResponse;
import ch.unisg.ics.interactions.wot.td.io.TDGraphReader;
import ch.unisg.ics.interactions.wot.td.schemas.DataSchema;
import ch.unisg.ics.interactions.wot.td.schemas.ObjectSchema;
import ch.unisg.ics.interactions.wot.td.schemas.StringSchema;
import ch.unisg.ics.interactions.wot.td.security.APIKeySecurityScheme;
import ch.unisg.ics.interactions.wot.td.vocabularies.TD;
import org.springframework.context.annotation.Primary;
import org.springframework.stereotype.Component;

import ch.unisg.executorrobot.executor.application.port.out.InstructionToRobotPort;

@Component
@Primary
public class InstructionToRobotAdapter implements InstructionToRobotPort {

    @Override
    public boolean instructionToRobot(String key) {

        String endpoint = "https://api.interactions.ics.unisg.ch/search/searchEngine";

        String input = "@prefix dct: <http://purl.org/dc/terms/> . select ?title where { ?title dct:title 'leubot1' }";

        var httpRequest = HttpRequest.newBuilder()
            .uri(URI.create(endpoint))
            .header("Content-Type", "application/json")
            .POST(HttpRequest.BodyPublishers.ofString(input))
            .build();

        var client = HttpClient.newHttpClient();

        try {
            var description = client.send(httpRequest, HttpResponse.BodyHandlers.ofString()).body();

            // Parse a TD from a string
            ThingDescription td = TDGraphReader.readFromString(ThingDescription.TDFormat.RDF_TURTLE, description);

            // Create the payload to be sent with the Http request
            Map<String, Object> elbowPayload = new HashMap<>();
            elbowPayload.put("http://www.w3.org/2001/XMLSchema#int", 400);

            // Get the affordance "setElbow" from the TD
            Optional<ActionAffordance> action = td.getActionByName("setElbow");

            // Get the first form
            if (action.isPresent()) {
                Optional<Form> form = action.get().getFirstForm();

                // Retrieve the input data schema from the action affordance
                Optional<DataSchema> inputSchema = action.get().getInputSchema();

                // If a form is found, use it to create and execute the Http request
                if (form.isPresent()) {
                    TDHttpRequest request = new TDHttpRequest(form.get(), TD.invokeAction);

                    if (inputSchema.isPresent()) {
                        request.setObjectPayload((ObjectSchema) inputSchema.get(), elbowPayload);

                        try {
                            TDHttpResponse response = request.execute();
                            System.out.println("Received response with status code: " + response.getStatusCode());
                            return true;
                        } catch (IOException e) {
                            e.printStackTrace();
                        }
                    }
                }
            }

        } catch (IOException e) {
            e.printStackTrace();
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
        return false;




    /*
        String putEndpoint = "https://api.interactions.ics.unisg.ch/leubot1/v1.3.0/elbow";

        String inputJson = "{ \"value\": 400}";
        var request = HttpRequest.newBuilder()
            .uri(URI.create(putEndpoint))
            .header("Content-Type", "application/json")
            .header("X-API-KEY", key)
            .PUT(HttpRequest.BodyPublishers.ofString(inputJson))
            .build();

        var client = HttpClient.newHttpClient();

        try {
            var response = client.send(request, HttpResponse.BodyHandlers.ofString());
            System.out.println(response.statusCode());
            System.out.println(response.headers());
            return true;
        } catch (IOException e) {
            e.printStackTrace();
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
        return false;
    */
    }

}
