package ch.unisg.executorrobot.executor.adapter.out;
import java.io.IOException;
import java.net.URI;
import java.net.http.HttpClient;
import java.net.http.HttpRequest;
import java.net.http.HttpResponse;
import java.util.HashMap;
import java.util.Map;
import java.util.Optional;

import ch.unisg.ics.interactions.wot.td.ThingDescription;
import ch.unisg.ics.interactions.wot.td.affordances.ActionAffordance;
import ch.unisg.ics.interactions.wot.td.affordances.Form;
import ch.unisg.ics.interactions.wot.td.clients.TDHttpRequest;
import ch.unisg.ics.interactions.wot.td.clients.TDHttpResponse;
import ch.unisg.ics.interactions.wot.td.io.TDGraphReader;
import ch.unisg.ics.interactions.wot.td.schemas.DataSchema;
import ch.unisg.ics.interactions.wot.td.schemas.ObjectSchema;
import ch.unisg.ics.interactions.wot.td.vocabularies.TD;
import org.springframework.boot.autoconfigure.sendgrid.SendGridAutoConfiguration;
import org.springframework.context.annotation.Primary;
import org.springframework.stereotype.Component;

import ch.unisg.executorrobot.executor.application.port.out.UserToRobotPort;

@Component
@Primary
public class UserToRobotAdapter implements UserToRobotPort {

    @Override
    public String userToRobot() {

        String endpoint = "https://api.interactions.ics.unisg.ch/search/searchEngine";

        String input = "@prefix dct: <http://purl.org/dc/terms/> . select ?title where { ?title dct:title 'Mirogate' }";

        var httpRequest = HttpRequest.newBuilder()
            .uri(URI.create(endpoint))
            .header("Content-Type", "application/json")
            .POST(HttpRequest.BodyPublishers.ofString(input))
            .build();

        var client = HttpClient.newHttpClient();

        try {
            var description = client.send(httpRequest, HttpResponse.BodyHandlers.ofString()).body();

            String uri = "http://yggdrasil.interactions.ics.unisg.ch/environments/61/workspaces/102/artifacts/leubot1";

            // Parse a TD from a string
            ThingDescription td = TDGraphReader.readFromURL(ThingDescription.TDFormat.RDF_TURTLE, uri);

            // Create the payload to be sent with the HTTP request
            Map<String, Object> logInPayload = new HashMap<>();
            logInPayload.put("http://xmlns.com/foaf/0.1/Name", "keanu rahimian");
            logInPayload.put("http://xmlns.com/foaf/0.1/Mbox", "keanu.rahimian@student.unisg.ch");

            // Get the affordance "Log-In" from the TD
            Optional<ActionAffordance> action = td.getActionByName("logIn");

            // Get the first form
            if (action.isPresent()) {
                Optional<Form> form = action.get().getFirstForm();

                // Retrieve the input data schema from the action affordance
                Optional<DataSchema> inputSchema = action.get().getInputSchema();

                // If a form is found, use it to create and execute the HTTP request
                if (form.isPresent()) {
                    TDHttpRequest request = new TDHttpRequest(form.get(), TD.invokeAction);

                    if (inputSchema.isPresent()) {
                        request.setObjectPayload((ObjectSchema) inputSchema.get(), logInPayload);

                        try {
                            TDHttpResponse response = request.execute();
                            System.out.println("Received response with status code: " + response.getStatusCode());

                            // TODO: Get the key from the response and return it
                            // Not exactly sure how to get the headers from the payload, as we need them
                            // to get the key

                            return null;

                        } catch (IOException e) {
                            e.printStackTrace();
                        }
                    }
                }
            }
        } catch (IOException e) {
            e.printStackTrace();
        } catch (InterruptedException e) {
            e.printStackTrace();
        }

    /*
        String inputJson = "{ \"name\":\"keanu rahimian\", \"email\":\"keanu.rahimian@student.unisg.ch\"}";
        var request = HttpRequest.newBuilder()
            .uri(URI.create(postEndpoint))
            .header("Content-Type", "application/json")
            .POST(HttpRequest.BodyPublishers.ofString(inputJson))
            .build();

        try {
            var response = client.send(request, HttpResponse.BodyHandlers.ofString());
            System.out.println(response.statusCode());
            System.out.println(response.headers());
            String url = response.headers().map().get("location").toString();
            String key = url.split("/")[url.split("/").length-1].split("]")[0];
            return key;
        } catch (IOException e) {
            e.printStackTrace();
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
        return null;
    */


        return null;
    }

}
